package emc.animation;

import java.util.*;
import java.awt.*;
import java.applet.*;
import rcs.posemath.*;
import rcs.nml.*;
import rcs.utils.*;

import emc.nml.*;

class emcAnimationShape
{
  static final int BOX_TYPE = 1;
  static final int CYL_TYPE = 2;


  boolean invisibleFromTop = false;
  boolean invisibleFromFront = false;
  boolean invisibleFromSide = false;

  int type;
  Color c = Color.blue;
  PM_POSE pose = new PM_POSE();
}



class emcAnimationBox extends emcAnimationShape
{
  double length;
  double width;
  double height;
  
  public emcAnimationBox()
  {
    type = BOX_TYPE;
  }

  public String toString()
  {
    return "{ length="+length+", width="+width+", height="+height+", pose="+pose+" } ";
  }
}

class emcAnimationCyl extends emcAnimationShape
{
  double length;
  double radius;
  int axis;

  public emcAnimationCyl()
  {
    type = CYL_TYPE;
  }

  public String toString()
  {
    return "{ length="+length+", radius="+radius+", axis="+axis+", pose="+pose+" } ";
  }
}

public class emcAnimation extends Applet implements Runnable
{
  EMC_STAT emcStatus = null;
  NMLConnection emcStatusConnection = null;
  Vector ShapeList = null;
  Vector SortedTopShapeList = null;
  Vector SortedFrontShapeList = null;
  Vector SortedSideShapeList = null;
  static public boolean debug_on = false;

  emcAnimationBox XAxisBox = null;
  emcAnimationBox StockBox = null;
  emcAnimationBox SpindleBox = null;
  emcAnimationCyl Spindle = null;

 
  public void InitializeShapeList()
  {
    ShapeList = new Vector();


    emcAnimationBox box = new emcAnimationBox();
    
    // Grey tray at the base of minimill
    box = new emcAnimationBox();
    box.length = 10.0;
    box.width = 10.0;
    box.height = 2.0;
    box.pose.tran.x = 2.0;
    box.pose.tran.y = 2.0;
    box.pose.tran.z = 2.0;
    box.c = Color.gray;
    ShapeList.addElement(box);

   // Blue tower
    box = new emcAnimationBox();
    box.length = 2.0;
    box.width = 3.0;
    box.height = 10.0;
    box.pose.tran.x = 5.5;
    box.pose.tran.y = 11.0;
    box.pose.tran.z = 4.0;
    box.c = Color.blue;
    ShapeList.addElement(box); 
    
    // Black Y axis
    box = new emcAnimationBox();
    box.length = 10.0;
    box.width = 3.0;
    box.height = 1.0;
    box.pose.tran.x = 5.5;
    box.pose.tran.y = 1.0;
    box.pose.tran.z = 4.0;
    box.c = Color.darkGray;
    ShapeList.addElement(box); 

    // Black X axis
    box = new emcAnimationBox();
    box.length = 3.0;
    box.width = 12.0;
    box.height = 1.0;
    box.pose.tran.x = 1.0;
    box.pose.tran.y = 6.0;
    box.pose.tran.z = 5.0;
    box.c = Color.darkGray;
    XAxisBox = box;
    ShapeList.addElement(box); 

    // Blue Wax Stock
    box = new emcAnimationBox();
    box.length = 3.0;
    box.width =  3.0;
    box.height = 1.0;
    box.pose.tran.x = 5.5;
    box.pose.tran.y = 6.0;
    box.pose.tran.z = 6.0;
    box.c = Color.blue;
    StockBox = box;
    ShapeList.addElement(box); 


    // Black Z axis
    box = new emcAnimationBox();
    box.length = 1.0;
    box.width = 3.0;
    box.height = 9.0;
    box.pose.tran.x = 5.5;
    box.pose.tran.y = 10.0;
    box.pose.tran.z = 5.0;
    box.c = Color.darkGray;
    ShapeList.addElement(box); 


    // Blue Spindle Box
    box = new emcAnimationBox();
    box.length = 3.0;
    box.width = 3.0;
    box.height = 3.0;
    box.pose.tran.x = 5.5;
    box.pose.tran.y = 7.0;
    box.pose.tran.z = 9.0;
    box.c = Color.blue;
    box.invisibleFromTop = true;
    SpindleBox = box;
    ShapeList.addElement(box); 

    // Spindle 
    emcAnimationCyl cyl = new emcAnimationCyl();
    cyl.length = 2.0;
    cyl.radius = 0.25;
    cyl.axis = 3;
    cyl.pose.tran.x = 6.75;
    cyl.pose.tran.y = 8.25;
    cyl.pose.tran.z = 7.0;
    cyl.c = Color.lightGray;
    Spindle = cyl;
    ShapeList.addElement(cyl);
  }
    
  public void init()
  {
    try
      {
	emcStatusConnection = new NMLConnection(new emcStatDict(), "emcStatus", "emcAnimation", "emc.nml");

      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
    InitializeShapeList();
  }

  Thread updatePositionThread = null;

  public void start()
  {
    if(null == updatePositionThread)
      {
	updatePositionThread = new Thread(this);
	updatePositionThread.start();
      }
  }
  
  public void stop()
  {
    if(null != updatePositionThread)
      {
	updatePositionThread.stop();
	updatePositionThread = null;
      }
  }

  public void run()
  {
    try
      {
	while(true)
	  {
	    UpdatePosition();
	    Thread.sleep(100);
	  }
      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
  }

  double scale_x = 15.0;
  double scale_y = 15.0;
  double scale_z = 15.0;
  
  static final int SHOW_ALL = 0;
  static final int SHOW_TOP = 1;
  static final int SHOW_FRONT = 2;
  static final int SHOW_SIDE = 3;
  
  int side_to_show = SHOW_ALL;
  
  public boolean mouseDown(Event evt, int x, int y)
  {
    if(side_to_show != SHOW_ALL)
      {
	side_to_show = SHOW_ALL;
	 scale_x = 15.0;
	 scale_y = 15.0;
	 scale_z = 15.0;
	return true;
      }

    if(x < size().width / 2)
      {
	if( y < size().height /2)
	  {
	    side_to_show = SHOW_TOP;
	    scale_x = 30.0;
	    scale_y = 30.0;
	    scale_z = 30.0;
	    return true;
	  }
	else
	  {
	    side_to_show = SHOW_FRONT;
	    scale_x = 30.0;
	    scale_y = 30.0;
	    scale_z = 30.0;
	    return true;
	  }
      }
    else
      {
	side_to_show = SHOW_SIDE;
	scale_x = 30.0;
	scale_y = 30.0;
	scale_z = 30.0;
	return true;
      }

  }

  public void paintTop(Graphics g)
  {
    try
      {
	Dimension d = size();
	int viewx_offset = 0;
	int viewy_offset = d.height/2;
	if(side_to_show != SHOW_ALL)
	  {
	    viewy_offset = d.height;
	  }
	for(int i = 0; i < SortedTopShapeList.size(); i++)
	  {
	    emcAnimationShape shape= (emcAnimationShape) SortedTopShapeList.elementAt(i);
	    if(shape.invisibleFromTop)
	      {
		continue;
	      }
	     int x=0,y=0,height=0,width=0;
	    switch(shape.type)
	      {
	      case emcAnimationShape.BOX_TYPE:
		emcAnimationBox box = (emcAnimationBox) shape;
		if(debug_on)
		  {
		    System.out.println("PaintTop box="+box);
		  }
		x = (int) (viewx_offset + (shape.pose.tran.x *scale_x));
		y = (int) (viewy_offset - (shape.pose.tran.y *scale_y) - (box.length * scale_y) );
		height = (int) (box.length * scale_y);
		width = (int) (box.width * scale_x);
		g.setColor(shape.c);
		g.fillRect(x,y,width, height);
		g.setColor(Color.black);
		g.drawRect(x,y,width,height);
		break;
		
	      case emcAnimationShape.CYL_TYPE:
		emcAnimationCyl cyl = (emcAnimationCyl) shape;
		if(debug_on)
		  {
		    System.out.println("PaintTop cyl="+cyl);
		  }
		switch(cyl.axis)
		  {
		  case 1: // X axis
		    x = (int) (viewx_offset+ (shape.pose.tran.x *scale_x));
		    y = (int) (viewy_offset - (shape.pose.tran.y *scale_y) - (cyl.radius*2 * scale_y) );
		    height = (int) (cyl.radius*2 * scale_y);
		    width = (int) (cyl.length * scale_x);
		    g.setColor(shape.c);
		    g.fillRect(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawRect(x,y,width,height);
		    break;
	
		  case 2: // Y axis
		    x = (int) (viewx_offset+ (shape.pose.tran.x *scale_x));
		    y = (int) (viewy_offset - (shape.pose.tran.y *scale_y) - (cyl.length * scale_y) );
		    height = (int) (cyl.length * scale_y);
		    width = (int) (cyl.radius*2 * scale_x);
		    g.setColor(shape.c);
		    g.fillRect(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawRect(x,y,width,height);
		    break;

		  case 3: // Z axis
		    x = (int) (viewx_offset+ (shape.pose.tran.x *scale_x));
		    y = (int) (viewy_offset - (shape.pose.tran.y *scale_y) - (cyl.radius*2 * scale_y) );
		    height = (int) (cyl.radius*2 * scale_y);
		    width = (int) (cyl.radius*2 * scale_x);

		    g.setColor(shape.c);
		    g.fillOval(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawOval(x,y,width,height);
		    break;
		  }
		break;
		
	      default:
		break;
	      }
	    if(debug_on)
	      {
		System.out.println("PaintTop x="+x+", y="+y+", height="+height+", width="+width);
	      }
	  }
      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
  }

  public void paintFront(Graphics g)
  {
    try
      {
	Dimension d = size();
	for(int i = 0; i < SortedFrontShapeList.size(); i++)
	  {
	    emcAnimationShape shape= (emcAnimationShape) SortedFrontShapeList.elementAt(i);
	    if(shape.invisibleFromFront)
	      {
		continue;
	      }
	    int x=0,y=0,height=0,width=0;
	    switch(shape.type)
	      {
	      case emcAnimationShape.BOX_TYPE:
		emcAnimationBox box = (emcAnimationBox) shape;
		if(debug_on)
		  {
		    System.out.println("PaintFront box="+box);
		  }
		x = (int) (shape.pose.tran.x *scale_x);
		y = (int) ((d.height) - (shape.pose.tran.z *scale_z) - (box.height * scale_z));
		height = (int) (box.height * scale_z);
		width = (int) (box.width * scale_x);
		g.setColor(shape.c);
		g.fillRect(x,y,width, height);
		g.setColor(Color.black);
		g.drawRect(x,y,width,height);
		break;
		
	      case emcAnimationShape.CYL_TYPE:
		emcAnimationCyl cyl = (emcAnimationCyl) shape;
		if(debug_on)
		  {
		    System.out.println("PaintFront cyl="+cyl);
		  }
		switch(cyl.axis)
		  {
		  case 1: // X axis
		    x = (int) (shape.pose.tran.x *scale_x);
		    y = (int) ((d.height) - (shape.pose.tran.z *scale_z) - (cyl.radius*2 * scale_z) );
		    height = (int) (cyl.radius*2 * scale_y);
		    width = (int) (cyl.length * scale_x);
		    g.setColor(shape.c);
		    g.fillRect(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawRect(x,y,width,height);
		    break;
	
		  case 2: // Y axis
		    x = (int) (shape.pose.tran.x *scale_x);
		    y = (int) ((d.height) - (shape.pose.tran.z *scale_z) - (cyl.radius*2 * scale_z) );
		    height = (int) (cyl.radius*2 * scale_z);
		    width = (int) (cyl.radius*2 * scale_x);
		    g.setColor(shape.c);
		    g.fillOval(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawOval(x,y,width,height);
		    break;

		  case 3: // Z axis
		    x = (int) (shape.pose.tran.x *scale_x);
		    y = (int) ((d.height) - (shape.pose.tran.z *scale_z) - (cyl.length * scale_z) );
		    height = (int) (cyl.length * scale_z);
		    width = (int) (cyl.radius*2 * scale_x);
		    g.setColor(shape.c);
		    g.fillRect(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawRect(x,y,width,height);
		    break;
		  }
		break;
		
	      default:
		break;
	      }
	    if(debug_on)
	      {
		System.out.println("PaintFront x="+x+", y="+y+", height="+height+", width="+width);
	      }
	  }
      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
  }

  public void paintSide(Graphics g)
  {
    try
      {
	Dimension d = size();
	int viewx_offset = d.width/2;
	int viewy_offset = d.height;
	if(side_to_show != SHOW_ALL)
	  {
	    viewx_offset = 0;
	  }
	for(int i = 0; i < SortedSideShapeList.size(); i++)
	  {
	    emcAnimationShape shape= (emcAnimationShape) SortedSideShapeList.elementAt(i);
	    if(shape.invisibleFromSide)
	      {
		continue;
	      }
	    int x=0,y=0,height=0,width=0;
	    switch(shape.type)
	      {
	      case emcAnimationShape.BOX_TYPE:
		emcAnimationBox box = (emcAnimationBox) shape;
		if(debug_on)
		  {
		    System.out.println("PaintFront box="+box);
		  }
		x = (int) (viewx_offset + (shape.pose.tran.y *scale_y));
		y = (int) (viewy_offset - (shape.pose.tran.z *scale_z) - (box.height * scale_z));
		height = (int) (box.height * scale_z);
		width = (int) (box.length * scale_y);
		g.setColor(shape.c);
		g.fillRect(x,y,width, height);
		g.setColor(Color.black);
		g.drawRect(x,y,width,height);
		break;
		
	      case emcAnimationShape.CYL_TYPE:
		emcAnimationCyl cyl = (emcAnimationCyl) shape;
		if(debug_on)
		  {
		    System.out.println("PaintSide cyl="+cyl);
		  }
		switch(cyl.axis)
		  {
		  case 1: // X axis
		    x = (int) (viewx_offset + (shape.pose.tran.y *scale_y));
		    y = (int) (viewy_offset - (shape.pose.tran.z *scale_z) - (cyl.radius*2 * scale_z) );
		    height = (int) (cyl.radius*2 * scale_y);
		    width = (int) (cyl.radius*2 * scale_x);
		    g.setColor(shape.c);
		    g.fillOval(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawOval(x,y,width,height);
		    break;
	
		  case 2: // Y axis
		    x = (int) (viewx_offset + (shape.pose.tran.y *scale_y));
		    y = (int) (viewy_offset - (shape.pose.tran.z *scale_z) - (cyl.radius*2 * scale_z) );
		    height = (int) (cyl.radius*2 * scale_z);
		    width = (int) (cyl.length * scale_x);
		    g.setColor(shape.c);
		    g.fillRect(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawRect(x,y,width,height);
		    break;

		  case 3: // Z axis
		    x = (int) (viewx_offset + (shape.pose.tran.y *scale_y));
		    y = (int) (viewy_offset - (shape.pose.tran.z *scale_z) - (cyl.length * scale_z) );
		    height = (int) (cyl.length * scale_z);
		    width = (int) (cyl.radius*2 * scale_y);
		    g.setColor(shape.c);
		    g.fillRect(x,y,width, height);
		    g.setColor(Color.black);
		    g.drawRect(x,y,width,height);
		    break;
		
		  default:
		    break;
		  }
		break;
	      }
	    if(debug_on)
	      {
		System.out.println("PaintSide x="+x+", y="+y+", height="+height+", width="+width);
	      }
	  }
      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
  }
  
  public void paintCross(Graphics g)
  {
    try
      {
	Dimension d = size();
	g.setColor(Color.black);
	g.drawLine(0,d.height/2, d.width, d.height/2);
	g.drawLine(d.width/2,0, d.width/2, d.height);
      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
  }

  public void paint(Graphics g)
  {
    setBackground(Color.white);
    switch(side_to_show)
      {
      case SHOW_ALL:
	paintTop(g); 
	paintFront(g);
	paintSide(g);
	paintCross(g);
	break;
	
      case SHOW_TOP:
	paintTop(g);
	break;

      case SHOW_FRONT:
	paintFront(g);
	break;

      case SHOW_SIDE:
	paintSide(g);
	break;
      }
  }


  public void SortTopShapeList()
  {
    SortedTopShapeList = new Vector();
    
    for(int i = 0; i < ShapeList.size(); i++)
      {
	emcAnimationShape shape = (emcAnimationShape) ShapeList.elementAt(i);
	
	SortedTopShapeList.addElement(shape);
      }
    
    for(int i = 0; i < SortedTopShapeList.size(); i++)
      {
	emcAnimationShape first_shape = (emcAnimationShape) SortedTopShapeList.elementAt(i);
	double first_shape_top = 0;
	switch(first_shape.type)
	  {
	  case emcAnimationShape.BOX_TYPE:
	    emcAnimationBox box = (emcAnimationBox) first_shape;
	    first_shape_top = box.pose.tran.z + box.height;
	    break;
	 
	  case emcAnimationShape.CYL_TYPE:
		emcAnimationCyl cyl = (emcAnimationCyl) first_shape;
		if(cyl.axis == 3)
		  {
		    first_shape_top = cyl.pose.tran.z + cyl.length;
		  }
		else
		  {
		    first_shape_top = cyl.pose.tran.z + cyl.radius*2;
		  }
		
		break;
	        
	  default:
	    break;
	  }
	for(int j = i+1; j < SortedTopShapeList.size(); j++)
	  {
	    emcAnimationShape last_shape = (emcAnimationShape) SortedTopShapeList.elementAt(j);
	    double last_shape_top = 0;
	    switch(last_shape.type)
	      {
	      case emcAnimationShape.BOX_TYPE:
		emcAnimationBox box = (emcAnimationBox) last_shape;
		last_shape_top = box.pose.tran.z + box.height;
		break;

	      case emcAnimationShape.CYL_TYPE:
		emcAnimationCyl cyl = (emcAnimationCyl) last_shape;
		if(cyl.axis == 3)
		  {
		    last_shape_top = cyl.pose.tran.z + cyl.length;
		  }
		else
		  {
		    last_shape_top = cyl.pose.tran.z + cyl.radius*2;
		  }
		if(cyl.axis == 3)
		  {
		    last_shape_top = cyl.pose.tran.z + cyl.length;
		  }
		else
		  {
		    last_shape_top = cyl.pose.tran.z + cyl.radius*2;
		  }
		break;
	    
	      default:
		break;
	      }
	    if(last_shape_top < first_shape_top)
	      {
		SortedTopShapeList.setElementAt(first_shape, j);
		SortedTopShapeList.setElementAt(last_shape, i);
		first_shape = last_shape;
		first_shape_top = last_shape_top;
	      }
	  }
      }
  }

  public void SortFrontShapeList()
  {
    SortedFrontShapeList = new Vector();
    
    for(int i = 0; i < ShapeList.size(); i++)
      {
	emcAnimationShape shape = (emcAnimationShape) ShapeList.elementAt(i);
	
	SortedFrontShapeList.addElement(shape);
      }
    
    for(int i = 0; i < SortedFrontShapeList.size(); i++)
      {
	emcAnimationShape first_shape = (emcAnimationShape) SortedFrontShapeList.elementAt(i);
	double first_shape_front = 0;
	switch(first_shape.type)
	  {
	  case emcAnimationShape.BOX_TYPE:
	    emcAnimationBox box = (emcAnimationBox) first_shape;
	    first_shape_front = box.pose.tran.y;
	    break;

	  case emcAnimationShape.CYL_TYPE:
	    emcAnimationCyl cyl = (emcAnimationCyl) first_shape;
	    if(cyl.axis == 2)
	      {
		first_shape_front = cyl.pose.tran.y + cyl.length;
	      }
	    else
	      {
		first_shape_front = cyl.pose.tran.y + cyl.radius*2;
	      }
	    break;

	  default:
	    break;
	  }
	for(int j = i+1; j < SortedFrontShapeList.size(); j++)
	  {
	    emcAnimationShape last_shape = (emcAnimationShape) SortedFrontShapeList.elementAt(j);
	    double last_shape_front = 0;
	    switch(last_shape.type)
	      {
	      case emcAnimationShape.BOX_TYPE:
		emcAnimationBox box = (emcAnimationBox) last_shape;
		last_shape_front = box.pose.tran.y;
		break;
	
    
	      case emcAnimationShape.CYL_TYPE:
		emcAnimationCyl cyl = (emcAnimationCyl) last_shape;
		if(cyl.axis == 2)
		  {
		    last_shape_front = cyl.pose.tran.y + cyl.length;
		  }
		else
		  {
		    last_shape_front = cyl.pose.tran.y + cyl.radius*2;
		  }
		break;

	      default:
		break;
	      }
	    if(last_shape_front > first_shape_front)
	      {
		SortedFrontShapeList.setElementAt(first_shape, j);
		SortedFrontShapeList.setElementAt(last_shape, i);
		first_shape = last_shape;
		first_shape_front = last_shape_front;
	      }
	  }
      }
  }

  public void SortSideShapeList()
  {
    SortedSideShapeList = new Vector();
    
    for(int i = 0; i < ShapeList.size(); i++)
      {
	emcAnimationShape shape = (emcAnimationShape) ShapeList.elementAt(i);
	
	SortedSideShapeList.addElement(shape);
      }
    
    for(int i = 0; i < SortedSideShapeList.size(); i++)
      {
	emcAnimationShape first_shape = (emcAnimationShape) SortedSideShapeList.elementAt(i);
	double first_shape_side =0;
	switch(first_shape.type)
	  {
	  case emcAnimationShape.BOX_TYPE:
	    emcAnimationBox box = (emcAnimationBox) first_shape;
	    first_shape_side = box.pose.tran.x + box.width;
	    break;
	    
	  case emcAnimationShape.CYL_TYPE:
	    emcAnimationCyl cyl = (emcAnimationCyl) first_shape;
	    if(cyl.axis == 1)
	      {
		first_shape_side = cyl.pose.tran.x + cyl.length;
	      }
	    else
	      {
		first_shape_side = cyl.pose.tran.x + cyl.radius*2;
	      }
	    break;
	  default:
	    break;
	  }
	for(int j = i+1; j < SortedSideShapeList.size(); j++)
	  {
	    emcAnimationShape last_shape = (emcAnimationShape) SortedSideShapeList.elementAt(j);
	    double last_shape_side = 0;
	    switch(last_shape.type)
	      {
	      case emcAnimationShape.BOX_TYPE:
		emcAnimationBox box = (emcAnimationBox) last_shape;
		last_shape_side = box.pose.tran.x + box.width;
		break;
	    
	      case emcAnimationShape.CYL_TYPE:
		emcAnimationCyl cyl = (emcAnimationCyl) last_shape;
		if(cyl.axis == 1)
		  {
		    last_shape_side = cyl.pose.tran.x + cyl.length;
		  }
		else
		  {
		    last_shape_side = cyl.pose.tran.x + cyl.radius*2;
		  }
		break;

	      default:
		break;
	      }
	    if(last_shape_side < first_shape_side)
	      {
		SortedSideShapeList.setElementAt(first_shape, j);
		SortedSideShapeList.setElementAt(last_shape, i);
		first_shape = last_shape;
		first_shape_side = last_shape_side;
	      }
	  }
      }
  }
	    
	
  public boolean first_time = true;
  public void UpdatePosition()
  {
    try
      {
	EMC_STAT emcStatusTemp = null;
	emcStatusTemp = (EMC_STAT) emcStatusConnection.read();
	if(null != emcStatusTemp || first_time)
	  {
	    emcStatus = emcStatusTemp;
	    if(null != emcStatus)
	      {
		double x, y, z;
		x = emcStatus.motion.traj.position.tran.x/emcStatus.motion.traj.linearUnits; // X Position in Millimeters.
		y = emcStatus.motion.traj.position.tran.y/emcStatus.motion.traj.linearUnits; // Y Position in Millimeters.
		z = emcStatus.motion.traj.position.tran.z/emcStatus.motion.traj.linearUnits; // Z Position in Millimeters.

		XAxisBox.pose.tran.y = 2.0 - y /60;
		StockBox.pose.tran.y = 2.0 - y /60;
		StockBox.pose.tran.x = 2.0 - x /50;
		SpindleBox.pose.tran.z = 11.0 + z /82;
		Spindle.pose.tran.z = 9.0 + z /82;
	      }
	    SortTopShapeList();
	    SortFrontShapeList();
	    SortSideShapeList();
	    first_time = false;
	    repaint();
	  }
      }
    catch(Exception e)
      {
	e.printStackTrace();
      }
  }

  public static  void main(String args[])
  {
    emcAnimation animation = new emcAnimation();
    emcAnimationFrame f = new emcAnimationFrame("EMC Animation",animation);
    f.resize(750,550);
    
    animation.init();
    animation.start();
    animation.show();

    f.add(animation);
    f.show();
  }
}
  
