#!/bin/sh
# the next line restarts using emcsh \
exec plat/nonrealtime/bin/emcsh "$0" "$@"

# This is the first write of a script to set coordinate system values.

wm title . "Dual Arc Joint Setup"

set top [frame .frame -borderwidth 2 -relief raised]
label $top.l1 -text "Dual Arc Joint Setup Window"
pack $top.l1 -side top

set paramfilename [emc_ini "PARAMETER_FILE" "RS274NGC"]
if {[string length $paramfilename] == 0} {
    set paramfilename "emc.var"
}

# Sets the number of joint parameters
set numjoint "6"
 
# Sets the names of the joint parameters
set namejoint "OE_Diameter Oversize Start_Position End_Angle Tool_Diameter Feed_Rate"

# Put the param file into an invisible widget
set vartext [text $top.vartext]
$vartext config -state normal
$vartext delete 1.0 end
if {[catch {open $paramfilename} programin]} {
    return
} else {
    $vartext insert end [read $programin]
    catch {close $programin}
}
     
# Build the variable numbers and value entry widgets.
set right [frame $top.col]
set joint [frame $right.coord]
label $joint.name -text "Parameter "
label $joint.varnum -text "Var # "
label $joint.varval -text "Value "
grid $joint.name $joint.varnum $joint.varval -sticky news
for {set i 0} {$i < $numjoint} {incr i} {
    label  $joint.l$i -text [lindex $namejoint $i ]  -anchor e
    label $joint.l1$i -textvariable "num$i"  -anchor e
    entry $joint.e$i -textvariable val$i -takefocus 1
    grid $joint.l$i $joint.l1$i $joint.e$i -sticky news
    bind $joint.e$i <Down>  {shiftFocus 1}
    bind $joint.e$i <Up> {shiftFocus -1}
}
focus $joint.e0

# Build the control button widgets.
set buttons [frame $right.buttons]
button $buttons.b1 -text "Close" -width 8 -command {closeWindow}
button $buttons.b2 -text "Write" -width 8 -command {setVarValues}
grid $buttons.b2 $buttons.b1 -sticky nsew
pack $joint -side top -fill x -expand yes
pack $buttons -side bottom -fill x -expand yes
pack $right -side left -fill both
pack $top

# Initialize values
for {set i 0} {$i < $numjoint} {incr i} {
    set val$i 0
}

proc findVarNumbers {} {
    global coordsys vartext numjoint num0 num1 num2 num3 num4 num5
    scan [$vartext index end] %d nl
    for {set i 0} {$i < $numjoint} {incr i} {
        set num$i [expr $coordsys +$i]
        if { [$vartext search [set num$i]  end ] == ""} {
            for {set j 1} {$j < $nl} {incr j} {
                set thisline [$vartext get $j.0 $j.4]
                if {$thisline > [set num$i] }  {
                    $vartext mark set index "[expr $j.0 - 1.0]"
                    $vartext insert index "[set num$i]	0.000000\n"
                    return
                }
            }
        }
    }
 }

proc findVarValues {} {
    global vartext
    global numjoint val0 val1 val2 val3 val4 val5
    global num0 num1 num2 num3 num4 num5
    global numjoint oval0 oval1 oval2 oval3 oval4 oval5
    set locate "1.0"
    for {set i 0} {$i < $numjoint} {incr i} {
        set oval$i [set val$i]
        set locate [$vartext search [set num$i] 1.0 ]
        set locate [expr int($locate)]
        set valtemp [$vartext get $locate.4 "$locate.end"]
        set val$i [string trim $valtemp]
    }
 }

proc setVarValues {} {
    global vartext
    global numjoint val0 val1 val2 val3 val4 val5
    global num0 num1 num2 num3 num4 num5
    for {set i 0} {$i < $numjoint} {incr i} {
        set locate [$vartext search [set num$i] 1.0 ]
        $vartext mark set insert "$locate +5c"
        $vartext delete insert "insert lineend"
        $vartext insert insert [set val$i]
        loadVarFile
    }
}

proc shiftFocus {whichway} {
    global joint numjoint
    set window [focus]
    set last [string trim [string length $window]]
    set last [expr $last - 1]
    set windowx [string index $window $last]
    set windownext [expr $windowx + $whichway]
    if {$windownext == $numjoint} {
        set windownext 0
    } elseif {$windownext < 0} {
        set windownext [expr $numjoint -1]
    }
    focus [string range $window 0 [expr $last - 1]]$windownext
}

proc saveFile {name} {
    global vartext
    catch {file copy -force $name $name.bak}
    if {[catch {open $name w} fileout]} {
        puts stdout "can't save $name"
        return
    }
    puts $fileout [$vartext get 1.0 end]
    catch {close $fileout}
}

proc loadVarFile {} {
    global top paramfilename
    saveFile $paramfilename
    emc_task_plan_init
}

# loads parameter file into interpreter and closes window
proc closeWindow {} {
    loadVarFile
    destroy .
}

proc saveFile {name} {
    global vartext
    catch {file copy -force $name $name.bak}
    if {[catch {open $name w} fileout]} {
        puts stdout "can't save $name"
        return
    }
    puts $fileout [$vartext get 1.0 end]
    catch {close $fileout}
}

# set the initial coordinate system and find values.
set coordsys 1100
findVarNumbers


