#!/bin/sh
# the next line restarts using emcsh \
exec plat/nonrealtime/bin/emcsh "$0" "$@"

# This is the first cut of a part program loop for Don McLane.
# GPL Copyright 2001 Ray Henry <rehenry@up.net>
# Loop.tcl needs to be run as root to communicate with backplot or set x auth.

emc_set_wait none

if {[emc_estop] == "on"} {
    puts "can't start loop with estop on"
    exit
    destroy .
}

wm title . "EMC Loop"

set looptime 1000

set top [frame .frame ]

set main [frame $top.main ]
label $main.l2 -textvariable loopfile -width 40
label $main.l3 -text "Count :" -justify right -width 8
set count 0
entry $main.e1 -textvariable count -width 3 -justify right
pack $main.l2 $main.l3 $main.e1 -side left

set buttons [frame $top.buttons]

button $buttons.b2 -font {Helvetica -12 bold} -text "Start Loop" -command {runLoop} \
    -bg lightgray -width 10
button $buttons.b3 -text "Stop at End" -state disabled  -width 10 \
    -command {set loopit "no" ; $buttons.b3 configure -state disabled}
set erase 0
checkbutton $buttons.erase -text "Send Erase" -width 10
button $buttons.b4 -text "Kill" -width 5 -command {exit ; destroy . }
pack $buttons.b2 $buttons.b3 $buttons.erase $buttons.b4 -side left -fill both -expand yes
pack $main $buttons -fill both -expand yes
pack $top

set programEntry [emc_ini PROGRAM_PREFIX DISPLAY]
set programDirectory $programEntry

# find the name of the running tk gui
set guiname [emc_ini DISPLAY DISPLAY]

proc loopFileDialog {w ent} {
    global programDirectory
    set types {
        {"NC files" {.nc .ngc}}
        {"Text files" {.txt}}
        {"All files" *}
    }
    set file [tk_getOpenFile -filetypes $types -initialdir $programDirectory ]
    set programDirectory [file dirname $file]
    return $file
}

set lastline 0
set loopit "no"
set loopfile "none"

proc loadLoop {} {
    global loopfile buttons lastline
    set loopfile ["emc_program"]
    emc_wait done
    if {$loopfile == "none" } {
        set loopfile [loopFileDialog . xxx]
        emc_mode auto
        emc_wait done
        emc_open $loopfile
    }
    set lastline [findLastLine $loopfile]
    $buttons.b2 config -bg darkgreen -activebackground green \
        -fg white -activeforeground black
}

proc findLastLine {filename} {
    set ttext [text .text]
    if {[catch {open $filename} programin]} {
            return
    } else {
        $ttext insert end [read $programin]
        catch {close $programin}
    }
    set lastline 0
    # finds the last line in file including newlines
    scan [$ttext index end] %d nl
    # finds the program end line
    for {set i 1} {$i < $nl} {incr i} {
        set temp [string tolower [$ttext get $i.0 $i.end] ]
        if {[string match *m2*  $temp ] || [string match *\%*  $temp ] || [string match *m30*  $temp ]} {
            set lastline $i
        }
    }
    # EMC does not set activeline to the line containing the end code.
    set lastline [expr $lastline - 1]
    # EMC does not set a line number for starting %
    set temp [string tolower [$ttext get $i.0 $i.end] ]
    if {[string match *\%*  $temp ]} {
        set lastline [expr $lastline - 1]
    }
    return $lastline
    catch [destroy $ttext]
}

proc runLoop {} {
    global mainfilename restartnumber buttons loopit loopfile guiname erase count
    if {[emc_mode] != "auto"} {
        emc_mode auto
    }
    if {[emc_program] == "none"} {
        emc_open $loopfile
    }
    if {$erase == 1} {
        send $guiname {erasePlot}
    }
    emc_run
    $buttons.b2 config -state disabled
    $buttons.b3 config -state normal
    set loopit "yes"
}

loadLoop
set laststatus [emc_program_status]

proc loop {} {
    global programStatus laststatus looptime loopfile
    global lastline loopit top buttons guiname count

    if {[emc_estop] == "on"} {
        emc_abort
        set loopit "no"
        destroy "."
    }

    set tempstatus [emc_program_status]
    if {$tempstatus != $laststatus} {
        set laststatus $tempstatus
    }

    set activeLine [emc_program_line]
    if {$activeLine == $lastline && $tempstatus == "idle" } {
        if {$loopit == "yes"} {
            set count [expr $count + 1]
            runLoop
        } else {
            $buttons.b2 configure -state normal
        }
    }
    after $looptime loop
}

loop
