#!/bin/sh
# the next line restarts using emcsh \
exec plat/nonrealtime/bin/emcsh "$0" "$@"

# This is the second cut of a subroutine script for tkemc.
# Need to fix for a start/restart from tkemc and have this script pick it up.

wm title . "EMC Run Subroutine"

# looptime is a process variable that takes the place of pauses in nc programs.
set looptime 100

set top [frame .frame ]

set main [frame $top.main ]
label $main.l1 -text "Main Program :"  -anchor e
label $main.l2 -textvariable mainname -anchor w -width 15
label $main.l3 -text "Line Number :"  -anchor e
label $main.l4 -textvariable restartnumber  -anchor w
label $main.l5 -text "of"
label $main.l6 -textvariable lastlineMain -anchor e
set maintext $main.text
text $maintext -relief groove -borderwidth 2 -takefocus 0 -height 3 -width 20 \
    -wrap none
grid $main.l1 $main.l2 -sticky ew
grid $main.l3 $main.l4 $main.l5 $main.l6 -sticky ew
grid $maintext -sticky ew
grid configure $main.l2 -columnspan 3
grid configure $maintext -columnspan 4

set sub [frame $top.sub -borderwidth 2 ]
label $sub.l1 -text "Subroutine :"  -anchor e
label $sub.l2 -textvariable subname  -anchor w -width 16
label $sub.l3 -text "Line Number :"  -anchor e
label $sub.l4 -textvariable subnumber  -anchor w
label $sub.l5 -text "of"
label $sub.l6 -textvariable lastlineSub -anchor e
set subtext $sub.text
text $subtext -relief groove -takefocus 0 -height 3 -width 20 -borderwidth 2 \
    -wrap none
grid $sub.l1 $sub.l2 -sticky ew
grid $sub.l3 $sub.l4 $sub.l5 $sub.l6 -sticky ew
grid $subtext -sticky ew
grid configure $sub.l2 -columnspan 3
grid configure $subtext -columnspan 4

set buttons [frame $top.buttons]
button $buttons.b -font {Helvetica -12 bold} -text "Load" -command {loadMain} \
    -bg lightgray
button $buttons.b1 -font {Helvetica -12 bold} -text "Start" -command {runMain} \
    -bg lightgray
button $buttons.b2 -font {Helvetica -12 bold} -textvariable stoptext -width 10 \
    -command {subroutineStop} -bg red -activebackground #f84c50
button $buttons.b3 -text "Close" -width 5 -bg lightgray \
    -command {destroy .}

grid $buttons.b -row 0 -col 0 -sticky nsew
grid $buttons.b1 -row 1 -col 0 -sticky nsew
grid $buttons.b3 -row 2 -col 0 -sticky nsew
grid $buttons.b2 -row 0 -col 1 -sticky nsew
grid configure $buttons.b2 -rowspan 3

grid $main $sub $buttons -sticky nsew
pack $top

set restartnumber 0
set programEntry [emc_ini PROGRAM_PREFIX DISPLAY]
set programDirectory $programEntry
set mainfilename ""

emc_set_wait received

proc fileDialog {w ent} {
    global programDirectory
    set types {
        {"NC files" {.nc .ngc}}
        {"Text files" {.txt}}
        {"All files" *}
    }
    set file [tk_getOpenFile -filetypes $types -initialdir $programDirectory ]
    set programDirectory [file dirname $file]
    return $file
}

proc loadMain {} {
    global mainname mainfilename maintext buttons lastlineMain
    set temp [fileDialog . $maintext]
    set mainfilename $temp
    emc_mode auto
    if {$temp != ""} {
        emc_open $temp
    }
    set mainname [file tail $temp]
    $maintext config -state normal
    $maintext delete 1.0 end
    if {[catch {open $mainfilename} programin]} {
        return
    } else {
        $maintext insert end [read $programin]
        $maintext config -state disabled
        catch {close $programin}
        $buttons.b1 config -bg darkgreen -activebackground green \
            -fg white -activeforeground black
        scan [$maintext index end] %d nl
        for {set i 1} {$i < $nl} {incr i} {
            set temp [$maintext get $i.0 $i.end]
            if {[string first M2 $temp ] != -1} {
                set lastlineMain $i
            }
        }
    }
}

proc runMain {} {
    global mainfilename restartnumber buttons
    if {$mainfilename == ""} {
        loadMain
        return
    }
    set restartnumber 0
    emc_task_plan_init
    if {[emc_mode] != "auto"} {
        emc_mode auto
    }
    emc_run
    $buttons.b config -state disabled -fg darkgray
    $buttons.b1 config -state disabled -fg darkgray
}

proc restartMain {} {
    global mainfilename restartnumber looptime
    emc_abort
    emc_open $mainfilename
    emc_wait done
    emc_run [expr $restartnumber + 2]
}

proc endMain {} {
    global maintext mainfilename restartnumber activeLine buttons laststatus
    set restartnumber 0
    emc_abort
    $maintext see 1.0
    $buttons.b config -state normal -fg black
    $buttons.b1 config -state normal -fg white
}

set subnumber 0
proc runSub {} {
    global subfilename subnumber subtext lastlineSub
    emc_mode auto
    emc_open $subfilename
    set subnumber 0
    if {[catch {open $subfilename} programin]} {
        puts stdout "can't open $subfilename"
    } else {
        $subtext config -state normal
        $subtext insert end [read $programin]
        $subtext config -state disabled
        catch {close $programin}
        scan [$subtext index end] %d nl
        for {set i 1} {$i < $nl} {incr i} {
            set temp [$subtext get $i.0 $i.end]
            if {[string first M2 $temp ] != -1} {
                set lastlineSub $i
            }
        }
    }
    emc_run
}

proc endSub {} {
    global subfilename subnumber subtext subname laststatus lastlineSub
    $subtext config -state normal
    $subtext delete 1.0 end
    $subtext config -state disabled
    set subname ""
    set subnumber 0
    set lastlineSub 1
    restartMain
}

set stoptext "FEEDHOLD"
set feedoverride [emc_feed_override]
set lastfeedoverride 100
proc subroutineStop {} {
    global stoptext buttons
    global feedoverride lastfeedoverride
    set temp [emc_feed_override]
    if {$temp != 0} {set lastfeedoverride $temp}
    switch -- $stoptext {
        FEEDHOLD {
            set feedoverride 0
            emc_feed_override $feedoverride
            $buttons.b2 configure -bg green -activebackground lightgreen
            set stoptext "CONTINUE"
        }
        CONTINUE {
            set feedoverride $lastfeedoverride
            emc_feed_override $feedoverride
            $buttons.b2 configure -bg red -activebackground #f84c50
            set stoptext "FEEDHOLD"
        }
        default {
        }

    }
}

proc coordSys {coord} {
    emc_abort
    emc_mode mdi
    emc_mdi $coord
    emc_wait done
    runSub
}

set subfilename ""
set laststatus idle
set lastlineMain 1
set lastlineSub 1

proc loop {} {
    global mainfilename restartnumber subfilename subname subnumber subtext
    global zz maintext programDirectory activeLine
    global stoptext programStatus laststatus
    global lastlineMain lastlineSub looptime

    if {[emc_estop] == "on"} {
        emc_abort
        emc_open $mainfilename
        emc_mode manual
        destroy .
    }

    if {$stoptext == "CONTINUE"} {
        if {[emc_feed_override] > 0} {
            subroutineStop
        }
    }

    set tempstatus [emc_program_status]
    if {$tempstatus != $laststatus} {
        set laststatus $tempstatus
    }

    set activeLine [emc_program_line]
    if {[emc_program] == $mainfilename && $activeLine > $restartnumber} {
        set restartnumber $activeLine
        $maintext see $activeLine.0
        set mainstring [$maintext get $activeLine.0 $activeLine.end]
        if {[string first gosub $mainstring] != -1} {
            set locate2 [expr [string first ) $mainstring] - 1 ]
            set locate1 [string wordstart $mainstring [expr $locate2 - 5]]
            set temp [string range $mainstring $locate1 $locate2]
            set subname $temp
            set subfilename $programDirectory/$temp
            set nextLine [expr $activeLine + 1]
            set nextstring [$maintext get $nextLine.0 $nextLine.end]
            set locate3 [expr [string first ( $nextstring] + 1 ]
            set locate4 [expr [string first ) $nextstring] - 1 ]
            set dothis [string range $nextstring $locate3 $locate4]
            catch [eval $dothis]
        }
    } elseif {[emc_program] == $subfilename} {
        set subnumber $activeLine
        $subtext see $activeLine.0
    }
    if {$restartnumber >= $lastlineMain } {
        endMain
    }
    if {$subnumber >= $lastlineSub } {
        endSub
    }
    after $looptime loop
}

loop
